﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;

namespace BmpTo84c
{
    class Program
    {
        static void Main(string[] args)
        {
            const int LINE_MAX = 10;
            const int BLUE_MAX = 31;
            const int GREEN_MAX = 63;
            const int RED_MAX = 31;

            // intro
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("bmpto84c - by James Vernon - www.jvti.org");
            Console.ForegroundColor = ConsoleColor.DarkGreen;
            Console.WriteLine("Converts .bmp files to .asm files suitable for use in TI-84+C");
            Console.WriteLine("Z80 Assembly programs");
            Console.ResetColor();

            // check arguments
            if (args.Length != 1)
            {
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("USAGE: bmpto84c <inputfile> (no extension)");
                Console.WriteLine("eg. bmpto84c logo");
                Console.ResetColor();
                return;
            }

            // load bmp & open output file
            Bitmap bmp = new Bitmap(args[0] + ".bmp");
            TextWriter asm = new StreamWriter(args[0] + ".asm");
            int width = bmp.Width;
            int height = bmp.Height;
            int lineCnt = 0;

            // comment at start of output file
            asm.WriteLine("; converted to Z80 ASM format with bmpto84c by James Vernon");
            asm.WriteLine();

            // read the bmp and construct the asm file
            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    if (lineCnt == 0)
                    {
                        // if a new line, start it with the ".dw"
                        asm.Write(".db ");
                    }
                    else
                    {
                        // otherwise, just put a comma
                        asm.Write(",");
                    }
                    Color pixel = bmp.GetPixel(x, y);
                    // the 84+C stores pixel data in 16-bit format in the order BGR (blue,green,red)
                    byte blue = (byte)(((float)(pixel.B)) / 255 * BLUE_MAX);
                    byte green = (byte)(((float)(pixel.G)) / 255 * GREEN_MAX);
                    byte red = (byte)(((float)(pixel.R)) / 255 * RED_MAX);
                    ushort value = (ushort)((blue << 11) | (green << 5) | (red));
                    byte lo = (byte)value;
                    byte hi = (byte)(value >> 8);
                    asm.Write(hi + "," + lo);
                    lineCnt++;
                    if (lineCnt >= LINE_MAX)
                    {
                        // if we've written enough for this line, put in a line break and reset the counter
                        asm.Write("\n");
                        lineCnt = 0;
                    }
                }
            }

            asm.Write("\n\n");
            asm.WriteLine(".end");

            // close asm file
            asm.Close();
        }
    }
}
